{{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Simple Debug main to allow me to see if I can get SPI comms from MBED to Propeller
// using the Very simple and well documented libs from Kwabena W. Agyeman
// This was initially checked in as a direct version of his demo.
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}

CON

  mine = 1
                                      

VAR
  long report
  long spistack[32]
  word cmd, last_res
  byte ssci
    
PUB initLocalSPI
  report := $1230
  cmd := $0
  last_res := $8401
  addr_last_res := @last_res
  addr_report := @report
  if ssci > 0
    cogstop( ssci - 1 )
  'ssci := cognew( spiSlave, @spistack ) + 1
  ssci := cognew( @spiSample, @report ) + 1
  return ssci

Pub getVal
  return report

Pub getRtn
  return last_res

PRI spiSlave | sync
  sync := 0
  ' no chip select for now; mode 3 op (clk default as high, sample on rising edge [phase 1])
  report := $7FFF_0001
  cmd := 0
  dira[2..0] := %010 '0 input (mosi), 1 output (miso), 2 clk (sclk)
  last_res := $80A3
  last_res ><= 16
  outa[1] := last_res & 1
  repeat
    waitpeq( |< 2, |< 2, 0 ) ' wait for the rising clock edge
    sync := cnt
    waitpeq( %%0, |< 2, 0 ) ' wait for the falling edge 0
    sync := cnt - sync
    report := sync
  until sync > 8000
  ' now output 80A3 (as the first transition has been spotted)
  outa[1] := last_res & 1
  last_res >>= 1
  waitpeq( |< 2, |< 2, 0 ) ' wait for the rising clock edge
  cmd := (cmd << 1) | ina[0]
  repeat 15
    waitpeq( %%0, |< 2, 0 ) ' wait for the first clock transition to 0
    ' setup output
    outa[1] := last_res & 1
    last_res >>= 1
    waitpeq( |< 2, |< 2, 0 ) ' wait for the rising clock edge
    ' sample
    cmd := (cmd << 1) | ina[0]
  
  report := $FF0F
  repeat
    last_res ><= 16
    cmd := 0 
    repeat 16
      waitpeq( %%0, |< 2, 0 ) ' wait for the first clock transition to 0
      ' setup output
      outa[1] := last_res & 1
      last_res >>= 1
      waitpeq( |< 2, |< 2, 0 ) ' wait for the rising clock edge
      ' sample
      cmd := (cmd << 1) | ina[0]
    last_res := cmd      
    'report := cmd
  until report == $FFFFFF
  
DAT
              org       0
spiSample     mov       dira, #%010 ' 0 input (mosi) 1 ouput (miso), 2 clk (sclk)
              mov       sample, #0
              mov       next_out, last_res_sync_val 
              rev       next_out, #16
              or        outa, #%10
              ' wait for a high of more than 8000 system clocks (which is half a 5Khz clock cycle) so twice as long as the high at 10K which is the spin speed               
:sync_loop    waitpeq   state_high, #%100
              mov       next_in, cnt
              waitpeq   state_low, #%100
              mov       sample, cnt
              sub       sample, next_in
              cmp       sample, sync_time       wc      ' c_set if sample < sync_time
        if_c  jmp       #:sync_loop                     ' the delay was less the the sync delay
              shr       next_out, #1            wc ' move bit 0 t carry flag ( and set bit 1 of outa with the result)
        if_c  or        outa, #%10              
        if_nc andn      outa, #%10
              waitpeq   state_high, #%100
              mov       sample, ina
              shr       sample, #1              wc
              rcl       next_in, #1
              ' fetch 15 bits from bus started at the next negative edge
              mov       sample_count, #15
:capture_sync waitpeq   state_low, #%100 ' wait for the clock transition to 0
              shr       next_out, #1            wc ' move bit 0 t carry flag ( and set bit 1 of outa with the result)
        if_c  or        outa, #%10              
        if_nc andn      outa, #%10
              waitpeq   state_high, #%100
              mov       sample, ina
              shr       sample, #1              wc
              rcl       next_in, #1
              djnz      sample_count, #:capture_sync
              ' we are now in sync 
:output_loop  rdword    next_out, addr_last_res
              rev       next_out, #16
              mov       next_in, #0
              ' fetch 16 bits from bus started at the next negative edge
              mov       sample_count, #16
:inner_loop   waitpeq   state_low, #%100
              shr       next_out, #1            wc ' move bit 0 t carry flag ( and set bit 1 of outa with the result)
        if_c  or        outa, #%10              
        if_nc andn      outa, #%10
              waitpeq   state_high, #%100
              mov       sample, ina
              shr       sample, #1              wc
              rcl       next_in, #1
              djnz      sample_count, #:inner_loop
              wrword    next_in, addr_last_res                                    
              wrword    next_in, addr_report
              jmp       #:output_loop
              nop
                                                  
sample        long    0       
next_in       long      0       
next_out      long      0
sample_count  long      %0       
state_high    long      %100       
state_low     long      %000
addr_last_res long      0       
addr_report   long      0
last_res_sync_val       long    $80A3
sync_time     long $8000  
              fit       
    
{{

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                  TERMS OF USE: MIT License
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
// Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}